@@ -98,8 +98,6 @@ class Encoder {
 				return EncodeDate(obj);
 			} else if (PyTime_Check(obj)) {
 				return EncodeTime(obj);
-			} else if (PyLong_CheckExact(obj)) {
-				return EncodeLong(obj);
 			} else if (PyFloat_CheckExact(obj)) {
 				return EncodeFloat(obj);
 			} else if (obj == Py_True) {
@@ -124,13 +122,17 @@ class Encoder {
 				Encoder_AppendFast('l');
 				Encoder_AppendFast('l');
 				Encoder_RETURN_TRUE;
+			} else if (PyLong_Check(obj)) {
+				return EncodeLong(obj);
 			} else if (PyAnySet_Check(obj)) {
 				return EncodeIterable(obj);
 			} else if (PyObject_HasAttr(obj, toJsonMethodName)) {
 				return EncodeWithJsonMethod<false>(obj);
 			} else if (PyObject_IsInstance(obj, Module::State()->ItemsView)) {
 				return EncodeItemsView(obj);
-			} else if (PyObject_IsInstance(obj, Module::State()->UUID)) {
+			} else if (Module::State()->Enum.Check(obj)) {
+				return EncodeEnum<false>(obj);
+			} else if (Module::State()->UUID.Check(obj)) {
 				Encoder_EnsureCapacity(Encoder_EXTRA_CAPACITY);
 				Encoder_AppendFast('"');
 				IF_LIKELY (EncodeUUID(obj)) {
@@ -187,7 +189,9 @@ class Encoder {
 				return EncodeWithDefault<true>(obj);
 			} else if (Module::State()->Decimal.CheckExact(obj)) {
 				return EncodeDecimal(obj);
-			} else if (PyObject_IsInstance(obj, Module::State()->UUID)) {
+			} else if (Module::State()->Enum.Check(obj)) {
+				return EncodeEnum<true>(obj);
+			} else if (Module::State()->UUID.Check(obj)) {
 				return EncodeUUID(obj);
 			}
 
@@ -409,15 +413,13 @@ class Encoder {
 			{ \
 				ms_size = value; \
 				CHOUT *end_position = buffer.cursor + EncodeDT_MILIS_MAX_LENGTH; \
-				CHOUT *saved_end_position = end_position; \
 				CHOUT digit; \
 				do { \
 					digit = (48 + (ms_size % 10)); \
-					if (digit != '0' || end_position != saved_end_position) { \
-						*(--end_position) = digit; \
-					} \
-				} while ((ms_size /= 10) > 0); \
-				ms_size = saved_end_position - end_position; \
+					*(--end_position) = digit; \
+					ms_size /= 10; \
+				} while (end_position > buffer.cursor); \
+				ms_size = EncodeDT_MILIS_MAX_LENGTH; \
 				memmove(buffer.cursor, end_position, sizeof(CHOUT) * ms_size); \
 				buffer.cursor += ms_size; \
 			}
@@ -553,6 +555,23 @@ class Encoder {
 			Encoder_RETURN_TRUE;
 		}
 
+		template<bool isDictKey>
+		Encoder_FN(EncodeEnum) {
+			PyObject* value = PyObject_GetAttr(obj, Module::State()->STR_VALUE);
+			if (value != NULL) {
+				bool res;
+				if (isDictKey) {
+					res = __encode_dict_key(value);
+				} else {
+					res = Encode(value);
+				}
+				Py_DECREF(value);
+				return res;
+			} else {
+				Encoder_RETURN_FALSE;
+			}
+		}
+
 		Encoder_FN(EncodeUUID) {
 			PyObject* str = PyObject_Str(obj);
 			if (str != NULL) {
